@@ -22,8 +22,6 @@ import org.apache.flink.api.common.operators.MailboxExecutor;
 import org.apache.flink.api.common.operators.ProcessingTimeService;
 import org.apache.flink.api.common.serialization.SerializationSchema;
 import org.apache.flink.api.connector.sink2.Sink;
-import org.apache.flink.api.connector.sink2.StatefulSink;
-import org.apache.flink.api.connector.sink2.TwoPhaseCommittingSink;
 import org.apache.flink.connector.base.DeliveryGuarantee;
 import org.apache.flink.connector.kafka.MetricUtil;
 import org.apache.flink.metrics.Counter;
@@ -69,8 +67,8 @@ import static org.apache.flink.util.Preconditions.checkState;
  * @param <IN> The type of the input elements.
  */
 class KafkaWriter<IN>
-        implements StatefulSink.StatefulSinkWriter<IN, KafkaWriterState>,
-                TwoPhaseCommittingSink.PrecommittingSinkWriter<IN, KafkaCommittable> {
+        implements TwoPhaseCommittingStatefulSink.PrecommittingStatefulSinkWriter<
+                IN, KafkaWriterState, KafkaCommittable> {
 
     private static final Logger LOG = LoggerFactory.getLogger(KafkaWriter.class);
     private static final String KAFKA_PRODUCER_METRIC_NAME = "KafkaProducer";
@@ -191,7 +189,6 @@ class KafkaWriter<IN>
     @Override
     public void write(@Nullable IN element, Context context) throws IOException {
         checkAsyncException();
-
         final ProducerRecord<byte[], byte[]> record =
                 recordSerializer.serialize(element, kafkaSinkContext, context.timestamp());
         if (record != null) {
@@ -452,7 +449,9 @@ class KafkaWriter<IN>
                 }
 
                 // Checking for exceptions from previous writes
-                mailboxExecutor.submit(
+                // Notice: throwing exception in mailboxExecutor thread is not safe enough for
+                // triggering global fail over, which has been fixed in [FLINK-31305].
+                mailboxExecutor.execute(
                         () -> {
                             // Checking for exceptions from previous writes
                             checkAsyncException();
