@@ -24,11 +24,11 @@ import org.apache.flink.api.common.serialization.SerializationSchema;
 import org.apache.flink.api.common.typeinfo.TypeInformation;
 import org.apache.flink.api.common.typeutils.TypeSerializer;
 import org.apache.flink.connector.base.DeliveryGuarantee;
+import org.apache.flink.connector.kafka.sink.KafkaPartitioner;
 import org.apache.flink.connector.kafka.sink.KafkaSink;
 import org.apache.flink.connector.kafka.sink.KafkaSinkBuilder;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.datastream.DataStreamSink;
-import org.apache.flink.streaming.connectors.kafka.partitioner.FlinkKafkaPartitioner;
 import org.apache.flink.table.api.DataTypes;
 import org.apache.flink.table.connector.ChangelogMode;
 import org.apache.flink.table.connector.Projection;
@@ -59,6 +59,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Objects;
 import java.util.Properties;
+import java.util.regex.Pattern;
 import java.util.stream.Stream;
 
 import static org.apache.flink.util.Preconditions.checkNotNull;
@@ -114,14 +115,17 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
      */
     @Nullable private final String transactionalIdPrefix;
 
-    /** The Kafka topic to write to. */
-    protected final String topic;
+    /** The Kafka topics to allow for producing. */
+    protected final List<String> topics;
+
+    /** The Kafka topic pattern of topics allowed to produce to. */
+    protected final Pattern topicPattern;
 
     /** Properties for the Kafka producer. */
     protected final Properties properties;
 
     /** Partitioner to select Kafka partition for each item. */
-    protected final @Nullable FlinkKafkaPartitioner<RowData> partitioner;
+    protected final @Nullable KafkaPartitioner<RowData> partitioner;
 
     /**
      * Flag to determine sink mode. In upsert mode sink transforms the delete/update-before message
@@ -143,9 +147,10 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
             int[] keyProjection,
             int[] valueProjection,
             @Nullable String keyPrefix,
-            String topic,
+            @Nullable List<String> topics,
+            @Nullable Pattern topicPattern,
             Properties properties,
-            @Nullable FlinkKafkaPartitioner<RowData> partitioner,
+            @Nullable KafkaPartitioner<RowData> partitioner,
             DeliveryGuarantee deliveryGuarantee,
             boolean upsertMode,
             SinkBufferFlushMode flushMode,
@@ -166,7 +171,8 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
         // Mutable attributes
         this.metadataKeys = Collections.emptyList();
         // Kafka-specific attributes
-        this.topic = checkNotNull(topic, "Topic must not be null.");
+        this.topics = topics;
+        this.topicPattern = topicPattern;
         this.properties = checkNotNull(properties, "Properties must not be null.");
         this.partitioner = partitioner;
         this.deliveryGuarantee =
@@ -206,7 +212,8 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
                         .setKafkaProducerConfig(properties)
                         .setRecordSerializer(
                                 new DynamicKafkaRecordSerializationSchema(
-                                        topic,
+                                        topics,
+                                        topicPattern,
                                         partitioner,
                                         keySerialization,
                                         valueSerialization,
@@ -223,7 +230,7 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
                         ProviderContext providerContext, DataStream<RowData> dataStream) {
                     final boolean objectReuse =
                             dataStream.getExecutionEnvironment().getConfig().isObjectReuseEnabled();
-                    final ReducingUpsertSink<?> sink =
+                    final ReducingUpsertSink<?, ?> sink =
                             new ReducingUpsertSink<>(
                                     kafkaSink,
                                     physicalDataType,
@@ -250,8 +257,13 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
     @Override
     public Map<String, DataType> listWritableMetadata() {
         final Map<String, DataType> metadataMap = new LinkedHashMap<>();
-        Stream.of(WritableMetadata.values())
-                .forEachOrdered(m -> metadataMap.put(m.key, m.dataType));
+        for (WritableMetadata m : WritableMetadata.values()) {
+            if (topics != null && topics.size() == 1 && WritableMetadata.TOPIC.key.equals(m.key)) {
+                // When `topic` is a singleton list, TOPIC metadata is not writable
+                continue;
+            }
+            metadataMap.put(m.key, m.dataType);
+        }
         return metadataMap;
     }
 
@@ -272,7 +284,8 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
                         keyProjection,
                         valueProjection,
                         keyPrefix,
-                        topic,
+                        topics,
+                        topicPattern,
                         properties,
                         partitioner,
                         deliveryGuarantee,
@@ -306,7 +319,8 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
                 && Arrays.equals(keyProjection, that.keyProjection)
                 && Arrays.equals(valueProjection, that.valueProjection)
                 && Objects.equals(keyPrefix, that.keyPrefix)
-                && Objects.equals(topic, that.topic)
+                && Objects.equals(topics, that.topics)
+                && Objects.equals(String.valueOf(topicPattern), String.valueOf(that.topicPattern))
                 && Objects.equals(properties, that.properties)
                 && Objects.equals(partitioner, that.partitioner)
                 && Objects.equals(deliveryGuarantee, that.deliveryGuarantee)
@@ -327,7 +341,8 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
                 keyProjection,
                 valueProjection,
                 keyPrefix,
-                topic,
+                topics,
+                topicPattern,
                 properties,
                 partitioner,
                 deliveryGuarantee,
@@ -393,6 +408,20 @@ public class KafkaDynamicSink implements DynamicTableSink, SupportsWritingMetada
     // --------------------------------------------------------------------------------------------
 
     enum WritableMetadata {
+        TOPIC(
+                "topic",
+                DataTypes.STRING().notNull(),
+                new MetadataConverter() {
+                    private static final long serialVersionUID = 1L;
+
+                    @Override
+                    public Object read(RowData row, int pos) {
+                        if (row.isNullAt(pos)) {
+                            return null;
+                        }
+                        return row.getString(pos).toString();
+                    }
+                }),
         HEADERS(
                 "headers",
                 // key and value of the map are nullable to make handling easier in queries
